/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */
#include "winner.h"

int32_t _pokerWinnersSorter(void *left, void* right) {
  return ((pokerwinning_t *)right)->type - ((pokerwinning_t *)left)->type;
}

void pokerWinnersSort(pokerwinning_t *winnings, int32_t length) {
  arraySort(sizeof(pokerwinning_t),
    winnings, length, (arraysort_t *)&_pokerWinnersSorter
  );
}


void pokerWinnerInit(poker_t *poker) {
  uint8_t i, j, number, highNumber;
  pokerwinning_t *left, *right;
  pokerplayer_t *player;
  card_t card, highCard;
  bool winner;

  poker->winnerCount = 0;

  for(i = 0; i < POKER_PLAYER_COUNT; i++) {
    left = poker->winnings + i;
    left->type = POKER_WINNING_TYPE_NULL;
    player = poker->players + i;
    if(!pokerPlayerIsAlive(player)) continue;

    // Get the players' full hand.
    pokerPlayerGetWinning(poker, player, left);
  }
  
  for(i = 0; i < POKER_PLAYER_COUNT; i++) {
    left = poker->winnings + i;
    if(left->type == POKER_WINNING_TYPE_NULL) continue;

    winner = true;
    highNumber = 0xFF;

    for(j = 0; j < POKER_PLAYER_COUNT; j++) {
      if(i == j) continue;

      right = poker->winnings + j;
      if(right->type == POKER_WINNING_TYPE_NULL) continue;

      // Am I the better hand?
      if(left->type < right->type) continue;
      if(left->type > right->type) {
        // No
        winner = false;
        break;
      }

      // Equal, compare hands.
      card = pokerPlayerCompareWinning(left, right);
      if(card == 0xFF) {
        winner = false;
        break;
      }

      number = cardGetNumber(card);
      if(highNumber == 0xFF || number == CARD_ACE || number > highNumber) {
        highCard = card;
        highNumber = number;
      }
    }

    if(!winner) continue;
    left->kicker = highCard;
    poker->winners[poker->winnerCount++] = i;
  }
  
  printf("Winner Count %u\n", poker->winnerCount);
}